#include "StdAfx.h"
#include "ConfigurationFileRetriever.h"


const string BASE_CONFIG_DIR =
	"../../../../../MasterThesis/DeviceConfigurator/res/cfg";

const string CONFIGURATION_FILENAME_HEAD = "config_dev";
const string CONFIGURATION_FILENAME_EXT = ".cfg";

const string INFORMATION_FILE_PATH_DEFAULT = 
	"../../../../../MasterThesis/DeviceConfigurator/res/references.info";

const string HISTORY_FILE_PATH_DEFAULT = 
	"../../../../../MasterThesis/DeviceConfigurator/res/referencesHistory.info";

const char INFORMATION_FILE_LINE_SEPARATOR = ';';
const char HISTORY_FILE_LINE_SEPARATOR = ';';


ConfigurationFileRetriever::ConfigurationFileRetriever()
{
	m_strInformationFilePath = INFORMATION_FILE_PATH_DEFAULT;
	m_strHistoryFilePath = HISTORY_FILE_PATH_DEFAULT;
}

ConfigurationFileRetriever::ConfigurationFileRetriever( const string informationFilePath,
														const string historyFilePath )
{
	m_strInformationFilePath = informationFilePath;
	m_strHistoryFilePath = historyFilePath;
}

ConfigurationFileRetriever::~ConfigurationFileRetriever()
{
}

DeviceConfiguratorStatus 
ConfigurationFileRetriever::GetConfigurationFilePath( DeviceInfo deviceInfo,
													 string& configurationFilePath )
{	
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_FAILED;
	configurationFilePath = "";

	string configurationFileReference = "";
	status = getConfigurationFileReference( deviceInfo, configurationFileReference ); 

	if ( status == DEVICE_CONFIGURATOR_OK )
	{
		configurationFilePath = BASE_CONFIG_DIR + "/" + configurationFileReference;
		return DEVICE_CONFIGURATOR_OK;
	}
	return status;
}



DeviceConfiguratorStatus 
ConfigurationFileRetriever::checkForNewerConfigurationFile( 
								const string& configurationFileReference,
								string& newerConfigurationFileReference )
{
	const int BUFFER_MAX_SIZE = 256;
	
	// search for configurationFileReference
	
	ifstream historyFileIfstream;
	
	// open history file
	historyFileIfstream.open( m_strHistoryFilePath.c_str() );
	if ( !historyFileIfstream.is_open() )
	{
		return DEVICE_CONFIGURATOR_NO_HISTORY_FILE;
	}
	
	// set newerConfigurationFileReference to current configurationFileReference
	newerConfigurationFileReference = configurationFileReference;

	char currentLine[BUFFER_MAX_SIZE];
	bool found = false;
	
	while( !historyFileIfstream.eof() && !found )
	{
		historyFileIfstream.getline( currentLine, BUFFER_MAX_SIZE );
		string strLine( currentLine );

		int startPos = strLine.find( configurationFileReference );
		
		if ( startPos == 0 )
		{
			// set newer value
			newerConfigurationFileReference = strLine.substr( 
								configurationFileReference.length() + 1 );
			found = true;
		}
	}

	// close history file 
	historyFileIfstream.close();
	
	return DEVICE_CONFIGURATOR_OK;
}

DeviceConfiguratorStatus 
ConfigurationFileRetriever::getConfigurationFileReference( DeviceInfo deviceInfo,
													 string& configurationFileReference )
{
	const int BUFFER_MAX_SIZE = 256;
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_FAILED;
	
	// create search strings - vendorString and deviceClassString
	char tmp[20];
	sprintf( tmp, "%d", deviceInfo.vendorId );
	string strVendorId = string( tmp );
	
	sprintf( tmp, "%d", deviceInfo.deviceId );
	if ( strlen( tmp ) < 2 )
	{
		return DEVICE_CONFIGURATOR_FAILED;
	}
	tmp[2] = '\0';
	string strDeviceClass = string( tmp );	
	string searchDescriptor = strVendorId.append( strDeviceClass );
	ifstream infoFileIfstream;
	
	// open information file (.info)
	infoFileIfstream.open( m_strInformationFilePath.c_str() );
	if ( !infoFileIfstream.is_open() )
	{
		return DEVICE_CONFIGURATOR_NO_INFORMATION_FILE;
	}
	
	char currentLine[BUFFER_MAX_SIZE];
	DeviceConfiguratorStatus currentResult = DEVICE_CONFIGURATOR_OK;
	bool found = false;
	string tmpConfigurationFileReference = "";

	while( !infoFileIfstream.eof() && !found
			&& currentResult == DEVICE_CONFIGURATOR_OK )
	{
		infoFileIfstream.getline( currentLine, BUFFER_MAX_SIZE );
		string descriptor = "";
		currentResult = getDescriptorFromLine( currentLine, descriptor );
		
		if ( currentResult == DEVICE_CONFIGURATOR_OK 
			&& descriptor.compare( searchDescriptor ) == 0 )
		{
			tmpConfigurationFileReference = getReferenceFromLine( currentLine );
			found = true;
		}
	}

	// close information file (.info)
	infoFileIfstream.close();
	
	if ( found )
	{
		status = checkForNewerConfigurationFile( tmpConfigurationFileReference,
												configurationFileReference );
		return status;
	}
	return DEVICE_CONFIGURATOR_FAILED;
}

DeviceConfiguratorStatus
ConfigurationFileRetriever::createConfigurationFilePathFromReference( 
											const string& configurationFileReference,
											string& configurationFilePath )
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_FAILED;
	int headStartPos = configurationFileReference.find( CONFIGURATION_FILENAME_HEAD );
	if ( headStartPos != 0 )
	{
		return DEVICE_CONFIGURATOR_FAILED;
	}

	int extStartPos = configurationFileReference.find( CONFIGURATION_FILENAME_EXT );
	if ( extStartPos != configurationFileReference.length() 
						- CONFIGURATION_FILENAME_EXT.length() )
	{
		return DEVICE_CONFIGURATOR_FAILED;
	}
	
	configurationFilePath = BASE_CONFIG_DIR + '/' + configurationFileReference;

	return DEVICE_CONFIGURATOR_OK;
}

DeviceConfiguratorStatus 
ConfigurationFileRetriever::getDescriptorFromLine( const char* line, string& descriptor )
{
	if ( line == NULL )
	{
		return DEVICE_CONFIGURATOR_FAILED;
	}

	int lineLength = strlen( line );
	int i = 0;
	while( i < lineLength && line[i] != INFORMATION_FILE_LINE_SEPARATOR )
	{
		i++;
	}

	if ( line[i] != INFORMATION_FILE_LINE_SEPARATOR )
	{
		return DEVICE_CONFIGURATOR_FAILED;
	}

	string strLine( line );

	// get string before i -th position
	descriptor = strLine.substr( 0, i );

	return DEVICE_CONFIGURATOR_OK;
}

string ConfigurationFileRetriever::getReferenceFromLine( const char* line )
{
	string reference = "";
	if ( line == NULL )
	{
		return reference;
	}

	int lineLength = strlen( line );
	int i = 0;
	while( i < lineLength && line[i] != INFORMATION_FILE_LINE_SEPARATOR )
	{
		i++;
	}

	if ( line[i] != INFORMATION_FILE_LINE_SEPARATOR 
		|| i == lineLength )
	{
		return reference;
	}

	string strLine( line );
	
	// get string after i -th position
	reference = strLine.substr( i+1 );

	return reference;
}

