#include "stdafx.h"

#include <cppunit/extensions/TestSuiteBuilder.h>
#include "ConfigurationFileRetrieverTestCase.h"
#include "TestIncludes.h"


const string CONFIGURATION_FILE_PATH_EXPECTED_RESULT = 
	"../../../../../MasterThesis/DeviceConfigurator/res/cfg/config_dev1234.cfg";

const string FILE_PATH_INVALID = "invalidFilePath";

const string CONFIGURATION_FILE_REFERENCE_OLD = "cfg1234";
const string CONFIGURATION_FILE_REFERENCE_NEW = "config_dev1234.cfg";


CU_TEST_SUITE_REGISTRATION( ConfigurationFileRetrieverTestCase );


void ConfigurationFileRetrieverTestCase::setUp()
{
	m_DeviceInfo.deviceId = DEVICE_ID_DEFAULT;
	m_DeviceInfo.vendorId = VENDOR_ID_DEFAULT;
}

void ConfigurationFileRetrieverTestCase::tearDown()
{
	
}

void ConfigurationFileRetrieverTestCase::testGetConfigurationFilePath()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;
	ConfigurationFileRetriever configurationFileRetriever;
	string configurationFilePath = "";
	status = configurationFileRetriever.GetConfigurationFilePath( m_DeviceInfo, 
															configurationFilePath );

	assert( status == DEVICE_CONFIGURATOR_OK );
	
	assert( configurationFilePath.compare( CONFIGURATION_FILE_PATH_EXPECTED_RESULT ) 
		== 0 );	
}

void ConfigurationFileRetrieverTestCase::testGetConfigurationFilePath_fail_noFileReference()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;
	ConfigurationFileRetriever configurationFileRetriever;
	string configurationFilePath = "";

	// set vendorId to unsupported vendor
	m_DeviceInfo.vendorId = UNSUPPORTED_VENDOR_ID;

	status = configurationFileRetriever.GetConfigurationFilePath( m_DeviceInfo, 
															configurationFilePath );

	assert( status != DEVICE_CONFIGURATOR_OK );
	
	assert( configurationFilePath.compare( "" ) == 0 );	
}

void ConfigurationFileRetrieverTestCase::testGetConfigurationFilePath_fail_noInformationFile()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;
	ConfigurationFileRetriever configurationFileRetriever;
	string configurationFilePath = "";

	// set invalid informationFilePath
	configurationFileRetriever.m_strInformationFilePath = FILE_PATH_INVALID;

	status = configurationFileRetriever.GetConfigurationFilePath( m_DeviceInfo, 
															configurationFilePath );

	assert( status == DEVICE_CONFIGURATOR_NO_INFORMATION_FILE );
	
	assert( configurationFilePath.compare( "" ) == 0 );		
}

void ConfigurationFileRetrieverTestCase::testCheckForNewerConfigurationFile()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;
	ConfigurationFileRetriever configurationFileRetriever;
	
	string newerConfigurationFileReference = "";
	status = configurationFileRetriever.checkForNewerConfigurationFile( 
												CONFIGURATION_FILE_REFERENCE_OLD, 
												newerConfigurationFileReference );

	assert( status == DEVICE_CONFIGURATOR_OK );
	
	assert( newerConfigurationFileReference.compare( CONFIGURATION_FILE_REFERENCE_NEW ) 
			== 0 );

	newerConfigurationFileReference = "";
	status = configurationFileRetriever.checkForNewerConfigurationFile( 
												CONFIGURATION_FILE_REFERENCE_NEW, 
												newerConfigurationFileReference );

	// newerConfigurationFileReference is latest and there is no record for it
	// in the history file, so it is valid and must be returned as "newer".
	assert( status == DEVICE_CONFIGURATOR_OK );
	
	assert( newerConfigurationFileReference.compare( CONFIGURATION_FILE_REFERENCE_NEW ) 
			== 0 );
}

void ConfigurationFileRetrieverTestCase::testCheckForNewerConfigurationFile_noHistoryFile()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;
	ConfigurationFileRetriever configurationFileRetriever;
	
	string newerConfigurationFileReference = "";

	// set invalid historyFilePath
	configurationFileRetriever.m_strHistoryFilePath = FILE_PATH_INVALID;

	status = configurationFileRetriever.checkForNewerConfigurationFile( 
												CONFIGURATION_FILE_REFERENCE_OLD, 
												newerConfigurationFileReference );

	assert( status == DEVICE_CONFIGURATOR_NO_HISTORY_FILE );
	
	assert( newerConfigurationFileReference.compare( "" ) == 0 );
}

void ConfigurationFileRetrieverTestCase::testGetConfigurationFileReference()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;
	ConfigurationFileRetriever configurationFileRetriever;
	
	string configurationFileReference = "";
	status = configurationFileRetriever.getConfigurationFileReference( 
												m_DeviceInfo, 
												configurationFileReference );

	assert( status == DEVICE_CONFIGURATOR_OK );
	
	assert( configurationFileReference.compare( CONFIGURATION_FILE_REFERENCE_NEW ) 
			== 0 );
}

void ConfigurationFileRetrieverTestCase::testGetConfigurationFileReference_fail_notExist()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;
	ConfigurationFileRetriever configurationFileRetriever;
	
	string configurationFileReference = "";

	// set deviceId to fail
	m_DeviceInfo.deviceId = 99;

	status = configurationFileRetriever.getConfigurationFileReference( 
												m_DeviceInfo, 
												configurationFileReference );

	assert( status == DEVICE_CONFIGURATOR_FAILED );
	
	assert( configurationFileReference.compare( "" ) == 0 );
}

void ConfigurationFileRetrieverTestCase::testCreateConfigurationFilePathFromReference()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;
	ConfigurationFileRetriever configurationFileRetriever;
	
	string configurationFilePath = "";
	status = configurationFileRetriever.createConfigurationFilePathFromReference( 
												CONFIGURATION_FILE_REFERENCE_NEW, 
												configurationFilePath );
	assert( status == DEVICE_CONFIGURATOR_OK );
	
	assert( configurationFilePath.compare( CONFIGURATION_FILE_PATH_EXPECTED_RESULT ) 
		== 0 );
}

void ConfigurationFileRetrieverTestCase::testCreateConfigurationFilePathFromReference_fail_invalidReference()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;
	ConfigurationFileRetriever configurationFileRetriever;
	
	string configurationFilePath = "";
	
	// call with invalid reference to fail
	status = configurationFileRetriever.createConfigurationFilePathFromReference( 
												CONFIGURATION_FILE_REFERENCE_OLD, 
												configurationFilePath );
	
	assert( status == DEVICE_CONFIGURATOR_FAILED );
	
	assert( configurationFilePath.compare( "" ) == 0 );
}

void ConfigurationFileRetrieverTestCase::testGetDescriptorFromLine()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;
	ConfigurationFileRetriever configurationFileRetriever;

	const char* line = "Descriptor;Other";
	const char* expectedDescriptor = "Descriptor";
	string resultDescriptor = "";
	status = configurationFileRetriever.getDescriptorFromLine( line, 
														resultDescriptor );

	assert( status == DEVICE_CONFIGURATOR_OK );
	
	assert( resultDescriptor.compare( expectedDescriptor ) == 0 );
}

void ConfigurationFileRetrieverTestCase::testGetDescriptorFromLine_fail_invalidLine()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;
	ConfigurationFileRetriever configurationFileRetriever;

	const char* invalidLine = "invalidLine";
	const char* expectedDescriptor = "";
	string resultDescriptor = "";
	status = configurationFileRetriever.getDescriptorFromLine( invalidLine, 
														resultDescriptor );

	assert( status == DEVICE_CONFIGURATOR_FAILED );
	
	assert( resultDescriptor.compare( expectedDescriptor ) == 0 );
}

void ConfigurationFileRetrieverTestCase::testGetReferenceFromLine()
{
	ConfigurationFileRetriever configurationFileRetriever;

	const char* line = "Descriptor;Reference";
	const char* expectedReference = "Reference";
	string resultReference = configurationFileRetriever.getReferenceFromLine( line );

	assert( resultReference.compare( expectedReference ) == 0 );
}

void ConfigurationFileRetrieverTestCase::testGetReferenceFromLine_fail_invalidLine()
{
	ConfigurationFileRetriever configurationFileRetriever;

	const char* invalidLine = "invalidLine";
	const char* expectedReference = "";
	string resultReference = configurationFileRetriever.getReferenceFromLine( 
																invalidLine );

	assert( resultReference.compare( expectedReference ) == 0 );
}