#include "stdafx.h"

#include <cppunit/extensions/TestSuiteBuilder.h>
#include "DeviceConfiguratorTestCase.h"
#include "TestIncludes.h"


CU_TEST_SUITE_REGISTRATION( DeviceConfiguratorTestCase );


void DeviceConfiguratorTestCase::setUp()
{
	m_DeviceInfo.deviceId = DEVICE_ID_DEFAULT;
	m_DeviceInfo.vendorId = VENDOR_ID_DEFAULT;
	m_LicenseInfo.deviceId = DEVICE_ID_DEFAULT;
	m_LicenseInfo.license = LICENSE_DEFAULT;
}

void DeviceConfiguratorTestCase::tearDown()
{
}

void DeviceConfiguratorTestCase::testRegisterClient()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;
	status = DC_RegisterClient( m_DeviceInfo );	
	
	// clean up without getting the status
	DC_UnregisterClient( m_DeviceInfo );

	assert( status == DEVICE_CONFIGURATOR_OK );
}

void DeviceConfiguratorTestCase::testUnregisterClient()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;
	status = DC_RegisterClient( m_DeviceInfo );
	assert( status == DEVICE_CONFIGURATOR_OK );

	status = DC_UnregisterClient( m_DeviceInfo );	
	assert( status == DEVICE_CONFIGURATOR_OK );
}

void DeviceConfiguratorTestCase::testUnregisterClient_fail_noSuchClient()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;
	status = DC_UnregisterClient( m_DeviceInfo );	
	assert( status == DEVICE_CONFIGURATOR_NO_SUCH_CLIENT );
}

void DeviceConfiguratorTestCase::testCheckLicense()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;
	status = DC_RegisterClient( m_DeviceInfo );	
	assert( status == DEVICE_CONFIGURATOR_OK );

	status = DC_CheckLicense( m_DeviceInfo, m_LicenseInfo );
	assert( status == DEVICE_CONFIGURATOR_OK );
	
	status = DC_UnregisterClient( m_DeviceInfo );	
	assert( status == DEVICE_CONFIGURATOR_OK );
}

void DeviceConfiguratorTestCase::testCheckLicense_fail_noSuchClient()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;

	// call without registering
	status = DC_CheckLicense( m_DeviceInfo, m_LicenseInfo );
	assert( status == DEVICE_CONFIGURATOR_NO_SUCH_CLIENT );
}

void DeviceConfiguratorTestCase::testCheckLicense_fail_invalidLicense()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;
	status = DC_RegisterClient( m_DeviceInfo );	
	assert( status == DEVICE_CONFIGURATOR_OK );

	// set an invalid license
	m_LicenseInfo.license = INVALID_LICENSE_DEFAULT;
	status = DC_CheckLicense( m_DeviceInfo, m_LicenseInfo );
	assert( status == DEVICE_CONFIGURATOR_INVALID_LICENSE );
	
	status = DC_UnregisterClient( m_DeviceInfo );	
	assert( status == DEVICE_CONFIGURATOR_OK );
}

void DeviceConfiguratorTestCase::testConfigureDevice()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;
	ConfigurationCommand* commands = NULL;
	int commandsCount = 0;

	status = DC_RegisterClient( m_DeviceInfo );	
	assert( status == DEVICE_CONFIGURATOR_OK );

	status = DC_ConfigureDevice( m_DeviceInfo, m_LicenseInfo, commands, commandsCount );
	assert( status == DEVICE_CONFIGURATOR_OK );
	assert( checkConfigurationCommands( commands, commandsCount ) );

	status = DC_FreeCommandData( commands, commandsCount );
	assert( status == DEVICE_CONFIGURATOR_OK );

	status = DC_UnregisterClient( m_DeviceInfo );	
	assert( status == DEVICE_CONFIGURATOR_OK );
}

void DeviceConfiguratorTestCase::testConfigureDevice_noSuchClient()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;
	ConfigurationCommand* commands = NULL;
	int commandsCount = 0;

	status = DC_ConfigureDevice( m_DeviceInfo, m_LicenseInfo, commands, commandsCount );
	assert( status == DEVICE_CONFIGURATOR_NO_SUCH_CLIENT );
}

void DeviceConfiguratorTestCase::testConfigureDevice_invalidLicense()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;
	ConfigurationCommand* commands = NULL;
	int commandsCount = 0;

	status = DC_RegisterClient( m_DeviceInfo );	
	assert( status == DEVICE_CONFIGURATOR_OK );

	// set an invalid license
	m_LicenseInfo.license = INVALID_LICENSE_DEFAULT;

	status = DC_ConfigureDevice( m_DeviceInfo, m_LicenseInfo, commands, commandsCount );
	
	assert( status == DEVICE_CONFIGURATOR_INVALID_LICENSE );

	// configuration commands must not be retrieved
	assert( !checkConfigurationCommands( commands, commandsCount ) );

	status = DC_FreeCommandData( commands, commandsCount );
	assert( status != DEVICE_CONFIGURATOR_OK );

	status = DC_UnregisterClient( m_DeviceInfo );	
	assert( status == DEVICE_CONFIGURATOR_OK );
}

void DeviceConfiguratorTestCase::testFreeCommandData()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;
	const int CONFIGURATION_COMMANDS_COUNT = 5;
	const int COMMAND_DATA_LENGTH = 50;
	const char* COMMAND_DATA = "TestData";

	ConfigurationCommand* configurationCommands = 
					new ConfigurationCommand[CONFIGURATION_COMMANDS_COUNT];

	if ( configurationCommands == NULL )
	{
		// fail
		assert( 0 );
	}

	for ( int i = 0; i < CONFIGURATION_COMMANDS_COUNT; i++ )
	{
		ConfigurationCommand* configurationCommand = configurationCommands + i;
		if ( configurationCommand != NULL )
		{
			configurationCommand -> data = new char[COMMAND_DATA_LENGTH];
			if ( configurationCommand -> data != NULL )
			{
				strcpy( configurationCommand -> data, COMMAND_DATA );
			}
		}
	}

	status = DC_FreeCommandData( configurationCommands, 
									CONFIGURATION_COMMANDS_COUNT );

	assert( status == DEVICE_CONFIGURATOR_OK );
}

void DeviceConfiguratorTestCase::testFreeCommandData_fail()
{
	DeviceConfiguratorStatus status = DEVICE_CONFIGURATOR_UNKNOWN_ERROR;
	
	status = DC_FreeCommandData( NULL, 0 );

	assert( status == DEVICE_CONFIGURATOR_FAILED );
}

bool DeviceConfiguratorTestCase::checkConfigurationCommands( 
										const ConfigurationCommand* commands,
										const int commandsCount )
{
	if ( commands == NULL || commandsCount <= 0 )
	{
		return false;
	}
	return true;
}
